/*
** Charge_Filter.c for Gmurf in ~/Gmurf/Charge_Filter
** 
** Made by 
** E-mail   <ahes@ker-ys.epita.fr>
** 
** Started on  Fri Mar 12 00:50:33 1999 
** Last update Mon Mar 15 17:21:59 1999 
*/

#include <fcntl.h>
#include <stdio.h>
#include "InOut.h"
#include <stdlib.h>
#include <unistd.h>
#include "Errors.h"
#include "Fct_Sys.h"
#include <gtk/gtk.h>
#include "Constants.h"
#include "Swap_File.h"
#include "Run_Filter.h"

#ifdef __NetBSD
#include <sys/wait.h>
#else
#include <wait.h>
#endif

/*
** new_tempgmurf_for_swap: Cree un fichier temporaire cible pour le filtre
*/
int	target_for_filter(name)
char	*name;
{
  int	result;


  result = open(name, O_RDWR | O_CREAT | O_APPEND | O_TRUNC, 0644);
  if (result == -1)
    {
#ifdef TEXT_MSG
      printf("%s\n", BAD_TEMP);
#endif
      gmurf_errno = BAD_CREAT;
      }
  return (result);
}

int	execute_filter(path, temp_path, target)
char	*path;
char	*temp_path;
char	*target;
{
  int	cpt;

  cpt = strlen(path) - 1;
  while ((cpt >= 0) && (path[cpt] != '/'))
    cpt--;
  if ((path[cpt] == '/') && (cpt > 0))
    {
      path[cpt] = '\0';
      chdir(path);
      path[cpt] = '/';
    }
  if (execlp (path, path, temp_path, target, swap_file, NULL) == -1)
     return (-1);
  return (1);
}


void		create_src_file(file, source, select, size)
int		file;
char		*source;
t_select	*select;
off_t		size;
{
  int			fd;
  off_t			nb;
  off_t			compt;
  off_t			begin;
  off_t			length;
  t_sound_sample	buf[BUFFER_SIZE];
  
  fd	 = open(source, O_CREAT | O_TRUNC | O_APPEND | O_RDWR, 0644);
  begin  = (select->reverse == 0 ? select->begin  : (select->begin == 0 ? select->begin + select->length : 0));
  length = (select->reverse == 0 ? select->length : size - select->length);
  compt = 0;
  lseek(file, begin * sizeof(t_sound_sample), SEEK_SET);
  while (compt < length)
    {
      nb = (compt + BUFFER_SIZE < length ? BUFFER_SIZE : length - compt);
      read(file, buf, nb * sizeof(t_sound_sample));
      write(fd, buf, nb * sizeof(t_sound_sample));
      compt += nb;
    }
  close(fd);
}

int		wav_re_create(select, temp_gmurf, source)
t_select	*select;
t_tempgmurf	*temp_gmurf;
char		*source;
{
  int			fd;
  int			trg;
  int			old;
  off_t			nb;
  off_t			size;
  off_t			compt;
  off_t			begin;
  off_t			length;
  t_sound_sample	*buf[BUFFER_SIZE];

  fd  = open(temp_gmurf->tempname, O_CREAT | O_TRUNC | O_APPEND | O_RDWR, 0644);
  old = open(undo_file, O_RDWR);
  trg = open(temp_file, O_RDWR);

  begin = 0;  
  length = (select->reverse == 0 ? select->begin : (select->begin == 0 ? select->length : 0)); 
  lseek(old, begin * sizeof(t_sound_sample), SEEK_SET); 
  compt = 0; 
  while (compt < length) 
    { 
      nb = (compt + BUFFER_SIZE < length ? BUFFER_SIZE : length - compt); 
      read(old, buf, nb * sizeof(t_sound_sample)); 
      write(temp_gmurf->file, buf, nb * sizeof(t_sound_sample)); 
      compt += nb; 
    } 
 
  length = give_file_size(trg) / sizeof(t_sound_sample); 
  compt = 0; 
  while (compt < length) 
    { 
      nb = (compt + BUFFER_SIZE < length ? BUFFER_SIZE : length - compt); 
      read(trg, buf, nb * sizeof(t_sound_sample)); 
      write(temp_gmurf->file, buf, nb * sizeof(t_sound_sample));
      compt += nb; 
    } 

  size   = give_file_size(old) / sizeof(t_sound_sample);
  begin  = (select->reverse == 0 ? select->begin + select->length : select->begin);  
  length = (select->reverse == 0 ? size - (select->begin + select->length) :  
  	    (select->begin == 0 ? 0 : select->length)); 
  lseek(old, begin * sizeof(t_sound_sample), SEEK_SET); 
  compt = 0; 
  while (compt < length) 
    { 
      nb = (compt + BUFFER_SIZE < length ? BUFFER_SIZE : length - compt); 
      read(old, buf, nb * sizeof(t_sound_sample)); 
      write(temp_gmurf->file, buf, nb * sizeof(t_sound_sample)); 
      compt += nb; 
    }
  close(trg);
  close(old);
  return (fd);
}


int		prepare_filter(path, filter_data, temp_gmurf, select)
char		*path;
t_filter_data	*filter_data;	
t_tempgmurf     *temp_gmurf;
t_select	*select;
{
  int	file_trg;
  int	plug_in_dead;
  char	*target;
  char	*source;
  char	plug_in_process_path[20];
  char	*pid_char;
  PID_T	pid;

  source = src_file;
  create_src_file(temp_gmurf->file, source, select, temp_gmurf->temp_size / sizeof(t_sound_sample));
  target = temp_file;
  if ((file_trg = target_for_filter(target)) == -1)
    return (-1);
  filter_data->temp_size = give_file_size_by_name(source);
  if (make_swap_file((void *) filter_data, sizeof(t_filter_data)) == -1)
    return (-1);
  close(temp_gmurf->file);
  if ((pid = fork()) == -1) 
    {
      gmurf_errno = BAD_FORK;
      return (-1);
    }
  /* ici on a un processus pere et un processus fils */
  if (pid == 0) /* si fils */
    {
      if (execute_filter(path, source, target) == -1)
	exit (-1);
      exit(1);
    }
  else		/* si pere */
    {
      strcpy(plug_in_process_path, "/proc/");
      pid_char = itoa(pid);
      strcat(plug_in_process_path, pid_char);
      strcat(plug_in_process_path, "/exe");
      free(pid_char);
      do
	{
	  plug_in_dead = access(plug_in_process_path, F_OK);
	  while(gtk_events_pending())
	    gtk_main_iteration();
	} while (plug_in_dead != -1);
      rename(temp_gmurf->tempname, undo_file);
      temp_gmurf->file = wav_re_create(select, temp_gmurf, source);
      temp_gmurf->position  = 0;
      temp_gmurf->temp_size = give_file_size(temp_gmurf->file);      
      unlink(source);
      unlink(temp_file);
      unlink(swap_file);
    }
  return (1);
}

int		exec_filter(path, filter_data, temp_gmurf, select)
char		*path;
t_filter_data	*filter_data;	
t_tempgmurf     *temp_gmurf;
t_select	*select;
{
  t_select	*sel;

  sel = (t_select *) malloc(sizeof(t_select));
  if (select->is_selected == 0)
    {
      sel->begin	= 0;
      sel->length	= temp_gmurf->temp_size / sizeof(t_sound_sample);
      sel->reverse	= 0;
      sel->chanel	= 2;
      sel->is_selected	= 1;
      prepare_filter(path, filter_data, temp_gmurf, sel);
    }
  else
    prepare_filter(path, filter_data, temp_gmurf, select);

  free(sel);

  return (1);
}
